import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot';

// URL to get the CSV data
const csvUrl = [
  'https://gist.githubusercontent.com/rakeshkommineni/3f88e5857303ef0cfe14f51dac207095/raw/fefc3205ee1fb912ce64c9b16cc605de5a6db932/Iris.csv'
].join('');

//  to parse each row in the CSV data
const parseRow = (d) => {
  d.sepal_length = +d.sepal_length;
  d.sepal_width = +d.sepal_width;
  d.petal_length = +d.petal_length;
  d.petal_width = +d.petal_width;
  return d;
};

// Declaring width and height of the SVG element
const width = window.innerWidth;
const height = window.innerHeight;

// Creating the SVG element and set their width and height
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

// Defining the main function
const main = async () => {
  // Calling the scatterPlot function and pass to the SVG element
  svg.call(
    // Setting the properties of the scatter plot
    scatterPlot()
      .width(width)
      .height(height)
      // Loading the CSV data and parsing it to use the parseRow function
      .data(await csv(csvUrl, parseRow))
      // Setting the x and y value functions for the scatter plot
      .xValue((d) => d.petal_width)
      .yValue((d) => d.sepal_length)
      // Setting the symbol value function  the scatter plot
      .symbolValue((d) => d.species)
      // Setting the margin of the scatter plot
      .margin({
        top: 20,
        right: 20,
        bottom: 40,
        left: 50,
      })
      // Setting the size of the symbols in the scatter plot
      .size(100)
  );
};

// Calling the main function here
main();
