//Importing  functions and objects from the D3 library
import {
  scaleLinear,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
  symbols,
  symbol,
} from 'd3';
// Define a factory function called scatterPlot, it will returns a new scatter plot function every time it is called 
export const scatterPlot = () => {
  // Declaring variables to store configuration settings for scatter plot
  let xValue;
  let yValue;let width;
  let height;
  let data;
  let symbolValue;
  let margin;
  let size;
 // Defining the scatter plot function that will be return by the factory-function
  const my = (selection) => {
    // Creating an x-scale and a y-scale using the D3 scaleLinear function
    const x = scaleLinear()
      .domain(extent(data, xValue))
     // Setting the range of the x-scale based on the width and margin settings
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
     // Setting the domain of the y-scale based on the yValue function and the data using the D3 extent function
      .domain(extent(data, yValue))
   // Setting the range of the y-scale based on the height and margin.
      .range([height - margin.bottom, 40]);
    
 // Creating an ordinal scale which maps symbol values to predefined symbols for using the D3 scaleOrdinal function
    const symbolScale = scaleOrdinal()
      .domain(data.map(symbolValue))
      .range(symbols);
 // Creating symbol generator function using the D3 symbol function
    const symbolGenerator = symbol().size(size);
 // Create an array of objects that contain the x and y positions, and path description for every data point    
    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      pathD: symbolGenerator.type(symbolScale(symbolValue(d)))()
    }));
    selection
      .append("text")   
        .attr("transform", "rotate(-90)")
        .attr("x", 0 - (height/2) )
        .attr("dy", "1em")
        .style("text-anchor", "middle")
        .text("sepal_width")
        .attr("font-family","times new roman");
      
  	selection.append('g')
			.attr('transform',
      `translate(0,
      ${height-margin.bottom})`)
			.call(axisBottom(x))
        .attr("font-family","Times new roman");
    
    // text label for the x axis
  	selection
      .append("text")
        .attr("x", width / 2 )
        .attr("y",  500)
        .style("text-anchor", "middle")
        .text("petal_width")
        .attr("font-family","times new roman");
                
    selection
      .append('text')
        .attr('class','axis-label')
        .attr('x',innerWidth/2.0)
        .attr('y',50)
        .style("font","30px times")
        .text("Scatter Plot");
 // Create SVG elements for each data point using the join method and position them using the translate method and the scales
    selection
      .selectAll('path')
      .data(marks)
      .join('path')
      .attr('d', (d) => d.pathD)
      .attr('transform', (d) => `translate(${d.x},${d.y})`);
 // Create an SVG axis for the y-scale and position it using the margin settings
    selection
      .append('g')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));
// Create an SVG axis for the x-scale and position it using the margin and height settings
    selection
      .append('g')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .call(axisBottom(x));
  };
// Defining setter functions for each configuration setting, which return the scatter plot function to allow the chaining
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.symbolValue = function (_) {
    return arguments.length
      ? ((symbolValue = _), my)
      : symbolValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.size = function (_) {
    return arguments.length ? ((size = +_), my) : size;
  };

  return my;
};
